#!/usr/bin/env python
# -*- coding: utf-8 -*-

#######################################################################
# maintainer: <schomi@vuplus-support.org>
# This plugin is free software, you are allowed to
# modify it (if you keep the license),
# but you are not allowed to distribute/publish
# it without source code (this version and your modifications).
# This means you also have to distribute
# source code of your modifications.
#######################################################################
# recode lululla

from . import _
from Components.ActionMap import ActionMap
from Components.AVSwitch import AVSwitch
from Components.ConfigList import ConfigListScreen
from Components.Label import Label
from Components.Pixmap import Pixmap
from Components.Sources.List import List
from Components.Sources.StaticText import StaticText
from Components.config import (
    config,
    ConfigSubsection,
    configfile,
    ConfigSelection,
    getConfigListEntry,
)
from Plugins.Plugin import PluginDescriptor
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Tools.Directories import SCOPE_PLUGINS, resolveFilename
from Tools.LoadPixmap import LoadPixmap
from enigma import eTimer, ePicLoad
from os import (
    listdir,
    remove,
    symlink,
    chdir,
    statvfs,
    path as os_path,
    walk as os_walk,
)
from os.path import join, getsize
import shutil

pname = _("PluginMover")
pdesc = _("Move plugins between flash memory and pen drive")
pversion = "0.2-r2"
pdate = "20140622"
# recode lululla


def bytes2human(n, digits=2):
    symbols = ('KB', 'MB', 'GB', 'TB', 'PB')
    prefix = {}
    for i, s in enumerate(symbols):
        prefix[s] = 1 << (i + 1) * 10
    for s in reversed(symbols):
        if n >= prefix[s]:
            if digits > 0:
                value = round((float(n) / prefix[s]), digits)
            else:
                value = n / prefix[s]
            return '%s %s' % (value, s)
    return "%s B" % n


def foldersize(size):
    try:
        fspace = round(float((size) // (1024.0 * 1024.0)), 2)
        spacestr = str(fspace) + 'MB'
        return spacestr
    except:
        return ""


def freespace(folder='/'):
    try:
        diskSpace = statvfs(folder)
        capacity = float(diskSpace.f_bsize * diskSpace.f_blocks)
        available = float(diskSpace.f_bsize * diskSpace.f_bavail)
        fspace = round(float((available) // (1024.0 * 1024.0)), 2)
        tspace = round(float((capacity) // (1024.0 * 1024.0)), 1)
        spacestr = 'Free space(' + str(fspace) + 'MB) Total space(' + str(tspace) + 'MB)'
        return spacestr
    except:
        return "location is unavaiable or not mounted"


def Plugins(**kwargs):
    return [PluginDescriptor(name=_("PluginMover"), description=_("Move your Plugins"), where=PluginDescriptor.WHERE_PLUGINMENU, icon="plugin.png", fnc=main),
            PluginDescriptor(name="PluginMover", description=_("Move your Plugins"), where=PluginDescriptor.WHERE_EXTENSIONSMENU, fnc=main)]


def main(session, **kwargs):
    print("[PluginMover]: Started ...")
    session.open(PluginMoverScreen)


def getMounts():
    devices = []
    with open('/proc/mounts', 'r') as f:
        for line in f:
            line = line.strip()
            entry = line.split()
            if entry[1][:7] == "/media/":
                devices.append(entry[1])
    return devices


# Config
devices = getMounts()
config.plugins.PluginMover = ConfigSubsection()
config.plugins.PluginMover.target = ConfigSelection(choices=devices)
# config.plugins.PluginMover.target = ConfigSelection(default="/media/usb", choices=devices)


class PluginMoverScreen(Screen):

    skin = """
        <screen position="center,center" size="1000,520" title="PluginMoverScreen" flags="wfNoBorder">
            <widget name="info" position="5,5" size="985,90" font="Regular;24" foregroundColor="#00fff000" />
            <widget source="menu" render="Listbox" position="15,100" size="630,350" zPosition="5" scrollbarMode="showOnDemand" transparent="1">
                <convert type="TemplatedMultiContent">
                    {"template":
                        [
                            MultiContentEntryPixmapAlphaTest(pos = (2, 2), size = (54, 54), png = 1),
                            MultiContentEntryText(pos = (60, 13), size = (100, 30), font=0, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER, text = 2),
                            MultiContentEntryText(pos = (170, 13), size = (450, 30), font=0, flags = RT_HALIGN_LEFT|RT_VALIGN_CENTER, text = 0),
                        ],
                        "fonts": [gFont("Regular", 24),gFont("Regular", 16)],
                        "itemHeight": 60
                    }
                </convert>
            </widget>
            <widget name="Picture" position="691,149" size="280,210" alphatest="on" />
            <ePixmap position="10,470" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_red.png" transparent="1" alphatest="on" />
            <ePixmap position="240,470" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_green.png" transparent="1" alphatest="on" />
            <ePixmap position="495,470" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_yellow.png" transparent="1" alphatest="on" />
            <widget source="key_red" render="Label" position="40,465" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
            <widget source="key_green" render="Label" position="275,465" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
            <widget source="key_yellow" render="Label" position="531,465" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
        </screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)
        self.skin_path = resolveFilename(SCOPE_PLUGINS, "Extensions/PluginMover")
        self.session = session
        self.title = pname + " (" + pversion + ")"
        try:
            self["title"] = StaticText(self.title)
        except:
            print('self["title"] was not found in skin')

        self["info"] = Label("Please wait..")
        self["Picture"] = Pixmap()
        self["key_red"] = StaticText(_("Exit"))
        self["key_green"] = StaticText(_("in Flash"))
        self["key_yellow"] = StaticText(_("Settings"))
        menu_list = []
        self["menu"] = List(menu_list)
        self["shortcuts"] = ActionMap(["SetupActions", "ColorActions", "DirectionActions"],
                                      {"ok": self.startmoving,
                                       "cancel": self.keyCancel,
                                       "red": self.keyCancel,
                                       "green": self.startmoving,
                                       "yellow": self.setup}, -2)

        self.plugin_base_dir = resolveFilename(SCOPE_PLUGINS, "Extensions")
        try:
            targetlocation = config.plugins.PluginMover.target.value
        except:
            targetlocation = "/media/usb"
        self.mount_dir = targetlocation
        self.ext_dir = targetlocation + "/Extensions"
        self.enabled_pic = LoadPixmap(cached=True, path=resolveFilename(SCOPE_PLUGINS, "Extensions/PluginMover/pic/loc_flash.png"))
        self.disabled_pic = LoadPixmap(cached=True, path=resolveFilename(SCOPE_PLUGINS, "Extensions/PluginMover/pic/loc_media.png"))
        if self.selectionChanged not in self["menu"].onSelectionChanged:
            self["menu"].onSelectionChanged.append(self.selectionChanged)
        self.PicLoad = ePicLoad()
        self.Scale = AVSwitch().getFramebufferScale()
        try:
            self.PicLoad.PictureData.get().append(self.DecodePicture)
        except:
            self.PicLoad_conn = self.PicLoad.PictureData.connect(self.DecodePicture)
        self.timer = eTimer()
        try:
            self.timer_conn = self.timer.timeout.connect(self.createMenuList)
        except:
            self.timer.callback.append(self.createMenuList)
        self.timer.start(100, True)

    def startmoving(self):
        self["info"].setText("Moving plugin,please wait...")
        self.timer = eTimer()
        try:
            self.timer_conn = self.timer.timeout.connect(self.runMenuEntry)
        except:
            self.timer.callback.append(self.runMenuEntry)
        self.timer.start(100, True)

    def selectionChanged(self, result=None):
        if result is True:
            self.createMenuList()
        try:
            sel = self["menu"].getCurrent()
        except:
            return
        try:
            self.setPicture(sel[0])
            if sel[1] == self.enabled_pic:
                self["key_green"].setText(_("in Flash"))
            elif sel[1] == self.disabled_pic:
                self["key_green"].setText(_("Exported"))
        except:
            pass
        self.getdevices_sizes()

    # Flash size
    def getdevices_sizes(self):
        freeflash = 'Unable to read flash size'

        try:
            # stat = statvfs("/")
            freeflash = freespace("/")
        except:
            pass

        # Device size
        freedev = "The device is unavailable or not mounted"
        try:
            devicelocation = config.plugins.PluginMover.target.value
            self.mount_dir = devicelocation
            freedev = devicelocation + ": " + freespace(devicelocation)
        except:
            try:
                devicelocation = config.plugins.PluginMover.target.value
            except:
                devicelocation = 'unknown'
            freedev = devicelocation + " " + freedev
            pass
        self["info"].setText(_("Flash: %s \nDevice: %s\n%s") % (freeflash, self.mount_dir, freedev))

    def createMenuList(self):
        self["info"].setText(_("read ..."))
        chdir(self.plugin_base_dir)
        f_list = []
        list_dir = sorted(listdir(self.plugin_base_dir))
        # print list_dir
        for f in list_dir:
            linked_dir = self.plugin_base_dir + "/" + f
            if os_path.isdir(linked_dir):
                free = self.GetFolderSize(linked_dir)
                size = str(foldersize(free))
                # pic
                pic = self.enabled_pic

                if os_path.exists(linked_dir):
                    if os_path.islink(linked_dir):
                        pic = self.disabled_pic
                    else:
                        pic = self.enabled_pic
                    f_list.append((f, pic, size))

        menu_list = []
        for entry in f_list:
            menu_list.append((entry[0], entry[1], str(entry[2])))
        self["menu"].updateList(menu_list)
        self.selectionChanged(False)

    def setPicture(self, f):
        plugin_dir = os_path.join(self.plugin_base_dir, f)
        preview = None
        if self["Picture"].instance:
            size = self['Picture'].instance.size()
            if size.isNull():
                size.setWidth(280)
                size.setHeight(210)
            for file in listdir(plugin_dir):
                if file.lower().endswith(".png"):
                    preview = os_path.join(plugin_dir, file)
                    break

            self.PicLoad.setPara([size.width(), size.height(), self.Scale[0], self.Scale[1], 0, 1, '#00000000'])
            if preview and os_path.exists(preview):
                if self.PicLoad.startDecode(preview):
                    print("Decodifica in corso:", preview)
                    self.PicLoad = ePicLoad()
                    try:
                        self.PicLoad.PictureData.get().append(self.DecodePicture)
                    except:
                        self.PicLoad_conn = self.PicLoad.PictureData.connect(self.DecodePicture)
                else:
                    print("Errore di decodifica dell'immagine.")
            return

    def DecodePicture(self, PicInfo=None):
        print('PicInfo=', PicInfo)
        ptr = self.PicLoad.getData()
        if ptr is not None:
            self["Picture"].instance.setPixmap(ptr)
            self["Picture"].instance.show()
        return

    def keyCancel(self):
        self.close()

    def runMenuEntry(self):
        try:
            devicelocation = config.plugins.PluginMover.target.value
            self.mount_dir = devicelocation
            print("242", self.mount_dir)
        except:
            pass
        if os_path.ismount(self.mount_dir):
            self["info"].setText(_("Moving, please wait ..."))
            sel = self["menu"].getCurrent()
            inflash = self.plugin_base_dir + "/" + sel[0]
            self.ext_dir = self.mount_dir + "/Extensions"
            notinflash = self.ext_dir + "/" + sel[0]
            # print "[PluginSkinMover] " + inflash
            # print "[PluginSkinMover] " + notinflash
            # print "[PluginMover] Start movement!"
            error = False
            if sel[1] == self.enabled_pic:
                if os_path.exists(notinflash):
                    shutil.rmtree(notinflash)
                # debug = True
                try:
                    shutil.copytree(inflash, notinflash)
                    error = False
                except:
                    error = True
                    # print "[PluginMover] Error during movement!"
                if error is False:
                    try:
                        shutil.rmtree(inflash)
                        symlink(notinflash, inflash)
                    except:
                        pass
            elif sel[1] == self.disabled_pic:
                if os_path.islink(inflash):
                    remove(inflash)
                    # debug = True
                    try:
                        shutil.copytree(notinflash, inflash)
                        error = False
                    except:
                        error = True
                        # print "[PluginMover] Error during movement!"
                    if error is False:
                        try:
                            shutil.rmtree(notinflash)
                        except:
                            pass
            if error:
                self.session.open(MessageBox, _("Plugin movement was not successful, please check devices!"), type=MessageBox.TYPE_ERROR, timeout=10)
                error = False

            self.createMenuList()
        else:
            self.session.open(MessageBox, _("No device to %s mounted. Plugin movement is not possible!") % self.mount_dir, type=MessageBox.TYPE_ERROR, timeout=10)
        self.selectionChanged()

    def GetFolderSize(self, path):
        TotalSize = 0
        for item in os_walk(path):
            for file in item[2]:
                try:
                    TotalSize = TotalSize + getsize(join(item[0], file))
                except:
                    print("error with file:  " + join(item[0], file))
        return TotalSize

    def setup(self):
        self.session.openWithCallback(self.setupCB, PluginMoverConfigScreen)

    def setupCB(self):
        self.selectionChanged()


class PluginMoverConfigScreen(Screen, ConfigListScreen):
    skin = """
        <screen position="center,center" size="1000,520" title="PluginMoverConfigScreen">
            <widget name="info" position="10,10" size="730,30" font="Regular;24" foregroundColor="#00fff000"/>
            <widget name="config" position="10,50" size="730,360" transparent="1" scrollbarMode="showOnDemand" />
            <ePixmap position="10,470" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_red.png" transparent="1" alphatest="on" />
            <ePixmap position="240,470" size="25,25" zPosition="0" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/PluginSkinMover/pic/button_green.png" transparent="1" alphatest="on" />
            <widget source="key_red" render="Label" position="40,465" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
            <widget source="key_green" render="Label" position="275,465" size="185,40" zPosition="1" font="Regular;24" halign="left" transparent="1" />
        </screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)
        self.session = session

        self["actions"] = ActionMap(["OkCancelActions", "ShortcutActions", "WizardActions", "ColorActions"], {
            "cancel": self.cancel,
            "red": self.cancel,
            "ok": self.save,
            "green": self.save
        }, -1)

        self['info'] = Label(_("Settings:"))
        # self['key_red'] = Label(_("Exit"))
        # self['key_green'] = Label(_("Ok"))
        self["key_red"] = StaticText(_("Exit"))
        self["key_green"] = StaticText(_("Ok"))

        self.list = []
        self.createConfigList()
        ConfigListScreen.__init__(self, self.list)

    def createConfigList(self):
        self.setTitle(pname + " (" + pversion + ")")
        self.list = []
        self.list.append(getConfigListEntry(_("Export Plugins to:"), config.plugins.PluginMover.target))

    def changedEntry(self):
        self.createConfigList()
        self["config"].setList(self.list)

    def save(self):
        for x in self["config"].list:
            x[1].save()
        config.save()
        configfile.save()
        self.close()

    def cancel(self):
        self.close()
